{% extends "!layout.html" %}

{% block header %}
	<!-- <img src="{{ pathto("_static/ars_logo.png", 1) }}" alt="ARS logo" />-->
{% endblock %}

{% block extrafooter %}
	<div class="footer">
        <!-- your html code here
        Hello!
        layout.html-->
    </div>
{% endblock %}