{% extends "!footer.html" %}

{% block extrafooter %}
	<div class="footer">
        <!-- your html code here -->
        <br>
        <img style="width: 200px; margin-left: auto; margin-right: auto; display: block;" src="{{ pathto("_static/ars_logo.png", 1) }}" alt="ARS logo" />
    </div>
{% endblock %}